@@ -3,40 +3,36 @@
 (* Created: September 2010 *)
 
 exception NotFlag of string
-exception NotBoolFlag of string
-exception NotIntFlag of string
 
-let bool_flags : (string,bool) Hashtbl.t = Hashtbl.create 20
-let int_flags : (string,int) Hashtbl.t = Hashtbl.create 50
+let use_learning : bool ref = ref false (*** use_learning is set to true if a command line option involving learning is included ***)
+let  bool_flags : (string, bool) Hashtbl.t = Hashtbl.create 20
+let   int_flags : (string,  int) Hashtbl.t = Hashtbl.create 50
 
-let get_bool_flag flagname = Hashtbl.find bool_flags flagname
-let set_bool_flag flagname flagval =
-  if (Hashtbl.mem bool_flags flagname) then
-    Hashtbl.replace bool_flags flagname flagval
-  else if (Hashtbl.mem int_flags flagname) then
-    raise (NotBoolFlag flagname)
-  else
-    raise (NotFlag flagname)
-let get_int_flag flagname = Hashtbl.find int_flags flagname
-let set_int_flag flagname flagval =
-  if (Hashtbl.mem int_flags flagname) then
-    Hashtbl.replace int_flags flagname flagval
-  else if (Hashtbl.mem bool_flags flagname) then
-    raise (NotIntFlag flagname)
-  else
-    raise (NotFlag flagname)
+let hashtbl_set tbl k v =
+  if (Hashtbl.mem tbl k) then Hashtbl.replace tbl k v
+  else raise (NotFlag k)
+
+let get_bool_flag = Hashtbl.find bool_flags
+let set_bool_flag = hashtbl_set bool_flags
+
+let get_int_flag = Hashtbl.find int_flags
+let set_int_flag = hashtbl_set int_flags
+
+let set_flag flagname l =
+  try set_bool_flag flagname (bool_of_string l)
+  with _ -> try Flags2.set_float_flag flagname (float_of_string l)
+  with _ -> set_int_flag flagname (int_of_string l)
 
 let print_flags () =
-  begin
-    Hashtbl.iter (fun x y -> Printf.printf "%s: %B\n" x y) bool_flags;
-    Hashtbl.iter (fun x y -> Printf.printf "%s: %d\n" x y) int_flags
-  end
+  Hashtbl.iter (fun x y -> Printf.printf "%s: %B\n" x y) bool_flags;
+  Hashtbl.iter (fun x y -> Printf.printf "%s: %d\n" x y) int_flags;
+  Flags2.print_float_flags ()
 
 let init_flags () =
-  begin
 (*** The following flags control search. ***)
     Hashtbl.add bool_flags "ENABLE_PATTERN_CLAUSES" false;
     Hashtbl.add bool_flags "PATTERN_CLAUSES_EARLY" false; (*** Dec 2012 - If true, make a pattern clause for the formula when it's assigned a literal instead of when it's processed. ***)
+    Hashtbl.add bool_flags "APPLY_PATTERN_CLAUSES_EARLY" false; (*** Nov 2015 - Resolve the pattern clause with other pattern clauses when it is created instead of when the corresponding proposition is processed. ***)
     Hashtbl.add bool_flags "DYNAMIC_PATTERN_CLAUSES" true; (*** Create pattern clauses from universally quantified formulas during search. - Chad, April 6, 2011 ***)
     Hashtbl.add bool_flags "PATTERN_CLAUSES_TRANSITIVITY_EQ" false; (*** Make pattern clauses for transitivity of equality. ***)
     Hashtbl.add bool_flags "PATTERN_CLAUSES_FORALL_AS_LIT" true;
@@ -150,6 +146,8 @@ let init_flags () =
     Hashtbl.add int_flags "HOUNIF1BOUND" 4;
     Hashtbl.add int_flags "HOUNIF1PROJECT" 1;
     Hashtbl.add int_flags "HOUNIF1IMITATE" 1;
+    Hashtbl.add bool_flags "HOUNIF1MATENONLITS" false; (*** Nov 2015 - When using HO Unif to suggest instantiations, try to mate on nonliterals. ***)
+    Hashtbl.add bool_flags "HOUNIF1NEGFLEX" false; (*** Nov 2015 - simplify dpairs like ~P... =? s where s is not a negation to be P... =? ~s; building double negation reduction into unification ***)
     Hashtbl.add bool_flags "EXISTSTOCHOICE" false; (*** Mar 2012 - Get rid of existentials during preprocessing ***)
 
 (*** The following flags should only be changed when trying to disprove ***)
@@ -171,10 +169,24 @@ let init_flags () =
     Hashtbl.add int_flags "E_PERIOD" 100;
     Hashtbl.add int_flags "E_TIMEOUT" 1;
     Hashtbl.add bool_flags "E_EQO_EQUIV" false;
+    Hashtbl.add bool_flags "E_AUTOSCHEDULE" false;
 
 (*** Feb 18 2013
  - INSTANTIATION_ORDER_CYCLE - how long to cycle a stack/queue switching pattern.
  - INSTANTIATION_ORDER_MASK - integer describing the stack/queue switching pattern. ***)
     Hashtbl.add int_flags "INSTANTIATION_ORDER_CYCLE" 1;
     Hashtbl.add int_flags "INSTANTIATION_ORDER_MASK" 0;
-  end
+
+(*** Nov 13 2015 DELAY_FO delay processing first-order formulas; with the idea that E or pattern clauses can handle this part of the search. ***)
+    Hashtbl.add int_flags "DELAY_FO" 0; (*** by default, no delay ***)
+    Hashtbl.add int_flags "DELAY_FO_CLAUSE" 0; (*** by default, no delay ***)
+    Hashtbl.add int_flags "DELAY_FO_LIT" 0; (*** by default, no delay ***)
+
+    
+    (*** Mar/Apr 2016 ***)
+    Hashtbl.add bool_flags "USE_MODELS" false;
+    Hashtbl.add int_flags "MAX_INTERPS_PER_AXIOM" 3; (*** for each axiom/negated conjecture, try to find MAX_INTERPS_PER_AXIOM interpretations of the constants in the frames with 1,2,4 base type elts making the axiom true; these will be used to help guide the search if USE_MODELS is true ***)
+    Hashtbl.add int_flags "DELAY_SEMANTIC_EQUIV_INSTANTIATION" 0; (*** by default, no delay, so don't bother evaluating in a model ***)
+    Hashtbl.add int_flags "DELAY_TRUE_IN_MODELS" 0; (*** delay processing a proposition based on how often it is true in a model, the idea being that those that are false are more likely to contribute to a refutation ***)
+    Hashtbl.add int_flags "DELAY_FALSE_IN_MODELS" 0; (*** delay processing a proposition based on how often it is false in a model ***)
+    Hashtbl.add int_flags "DELAY_UNINFORMATIVE_IN_MODELS" 0 (*** delay those that are true in all models or false in all models ***)
