@@ -1,5 +1,7 @@
 open Flags
 open State
+open Log
+open Atom
 open String
 open Syntax
 open Search
@@ -11,6 +13,8 @@ open Latex
 open Coq
 open Branch
 open Norm
+open Refut
+open Error
 
 (** to String functions for debugging**)
 
@@ -182,66 +186,111 @@ let (s2,_,w2,_,_,_,_) = statistic1 r 0 false in
 (** MAIN **)
 let tstp : bool ref = ref false;;
 
-let print_coq_proofscript c r =
-  
-  if (!verbosity > vold) then Printf.printf  "starting print_coq_proofscript.\n";
-  if (!verbosity > vold) then print_refut r;
 
-	(*** Search ***)
+let print_latex_search (size,_,depth,width,_,cut,rewrite,notyetImpl,_) lat c =
+  if (result_latex && width < 50 && depth < 30) then
+  Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d \n \n \\begin{tabular}{c} \n %s \\end{tabular} \n\n %%endLatex \n ***)\n" 
+  size depth width lat
+  else if (result_latex) then Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d \n \n  %%endLatex \n ***)\n" size depth width
 
-  (* get assumptions *)
-  let initbranch1 = (List.rev !initial_branch) in
-  let initbranch = (List.map preprocess initbranch1) in (*** Chad, Aug 2011: Preprocess to handle LEIBEQ_TO_PRIMEQ here as well as in suche.ml. Example: SYO348^5.p ***)
-  let b =List.map get_literal initbranch in
-  (* call search *)
-  let (refutation,con,searchTime,isTimeout) = search_refutation b r in
-  (* check wether the dependency of the returned refutation is a subset of the assumptions - fail indicates some bug*)
-  if assert_condition && not (Dependency.is_empty (Dependency.diffunion [] [con] [b]) ) then 
-    (if (!verbosity > 0) then Printf.printf  "Error with refutation: Still open conditions  \n") ;	 
+
+let print_latex_translation (size,_,depth,width,_,cut,rewrite,notyetImpl,_) lat c =
+  if (result_latex &&  width < 50 && depth < 30) then
+  Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d cuts %d rewrite  %d NYI %d \n \n \\begin{tabular}{c} \n %s \\end{tabular} \n\n %%endLatex \n***)" 
+  size depth width cut rewrite notyetImpl lat
+  else if (result_latex) then Printf.fprintf c "(*** \n %%beginLatex \n \n \n Translation successful, probably \n \n size %d depth %d width %d cuts %d rewrite %d  NYI %d \n %%endLatex \n***)" size depth width cut rewrite notyetImpl 
+  ; flush stdout
+
+
+let print_refutation_result (refutation,_,searchTime,isTimeout) =
   let (size,_,depth,width,_,cut,rewrite,notyetImpl,timeouts) = statistic refutation in
-  (* if timeout was reached the refutation is unfinished - at least output some information about the completed part*)
-	if isTimeout  || timeouts > 0 then  
-  	  (if (!verbosity > 0) then Printf.printf  "Timeout!  time %d ms size %d depth %d width %d cuts %d rewrite %d NYI %d timeouts %d \n" searchTime size depth width cut rewrite notyetImpl timeouts)
-  	else 	
+  let hasTimedout = isTimeout || timeouts > 0 in
+  if hasTimedout then (if !verbosity > 0 then Printf.printf  "Timeout!  time %d ms size %d depth %d width %d cuts %d rewrite %d NYI %d timeouts %d \n" searchTime size depth width cut rewrite notyetImpl timeouts)
+  else
   begin
     if result_print_search then Printf.printf  "%s\n" (ref_str refutation);
-    if (!verbosity > 3) then Printf.printf "Search time %d ms size %d depth %d width %d \n" searchTime size depth width ;
+    if (!verbosity > 3) then Printf.printf "Search time %d ms size %d depth %d width %d \n" searchTime size depth width;
+  end;
+  (refutation, hasTimedout)
+  
+let get_initbranch () =
+  let initbranch1 = (List.rev !initial_branch) in
+  List.map preprocess initbranch1 (*** Chad, Aug 2011: Preprocess to handle LEIBEQ_TO_PRIMEQ here as well as in suche.ml. Example: SYO348^5.p ***)
 
-	(*** Translation  ***)
+let get_initbranch_prenorm () =
+  let go pn =
+    let p = onlynegnorm pn in
+    if debug_translation then Printf.printf "%s = %s \n" (trm_str pn) (trm_str p); p in
+  List.map go (List.rev !initial_branch_prenorm)
+  
 
-(* get prenormalized assumptions *)
-  let initbranch_prenorm = List.map 
-	(fun pn ->let p= onlynegnorm pn in  if debug_translation then Printf.printf  "%s = %s \n" (trm_str pn) (trm_str p) ;p ) 
-	(List.rev !initial_branch_prenorm) in
-  let beforeTrans= Sys.time() in
-(* Branch is an association list from normalized terms to their prenormalized counterparts *)
-  let branch = (List.combine initbranch  initbranch_prenorm) in
-  if debug_translation then (Printf.printf "About to translate. Branch assoc with prenorm:\n"; List.iter (fun (m,p) -> Printf.printf "trm: %s\npre: %s\n" (trm_str m) (trm_str p)) branch);
-(* The flag pftrm_eager_rewrite decides wether rewrites are eagerly or lazily handled *) 
-  let prenorm_refutation =
-    begin
+let get_refutation initbranch r =
+  (* get assumptions *)
+  let b = List.map get_literal initbranch in
+  (* call search *)
+  let (_,con,_,_) as sr = search_refutation b r in
+  if assert_condition && not (Dependency.is_empty (Dependency.diffunion [] [con] [b])) && !verbosity > 0 then
+    Printf.printf "Error with refutation: Still open conditions\n";
+  sr
+
+let do_translate branch refutation =
       try
 	if pftrm_eager_rewrite then
 	  eager_translate branch refutation
 	else
 	  lazy_translate branch refutation
       with TranslateFailure -> safe_translate branch refutation (*** Chad Aug 2011 : Fall back on safe_translate ***)
-    end
-  in
+
+let get_translation branch refutation =
+  if debug_translation then (Printf.printf "About to translate. Branch assoc with prenorm:\n"; List.iter (fun (m,p) -> Printf.printf "trm: %s\npre: %s\n" (trm_str m) (trm_str p)) branch);
+(* The flag pftrm_eager_rewrite decides wether rewrites are eagerly or lazily handled *) 
+  let prenorm_refutation = do_translate branch refutation in
+
   if debug_translation then (Printf.printf "Translated prenorm_refutation\n%s\n" (ref_str prenorm_refutation));
-  let transTime= int_of_float ((Sys.time() -. beforeTrans) *. 1000.0) in
-  begin
-  let (size,_,depth,width,_,cut,rewrite,notyetImpl,_) = statistic prenorm_refutation  in
+  prenorm_refutation
+
+let print_prenorm_ref_result transTime stats prenorm_refutation =
+  let (size,_,depth,width,_,cut,rewrite,notyetImpl,_) as stats = statistic prenorm_refutation  in
   if result_print_translation then Printf.printf  "%s\n" (ref_str prenorm_refutation);
-  if (!verbosity > 3) then Printf.printf  "Translation NYI %d time %d ms size %d depth %d width %d cuts %d rewrite %d  \n" notyetImpl transTime size depth width cut rewrite  ;
+  if (!verbosity > 3) then Printf.printf  "Translation NYI %d time %d ms size %d depth %d width %d cuts %d rewrite %d  \n" notyetImpl transTime size depth width cut rewrite
+  
+  
+
+let print_coq_proof printer c r =
+  if (!verbosity > vold) then print_refut r;
+
+	(*** Search ***)
+
+  let initbranch = get_initbranch () in
+  let (refutation, hasTimedout) = print_refutation_result (get_refutation initbranch r) in
+
+  if hasTimedout then raise Timeout;
+	(*** Translation  ***)
+
+(* get prenormalized assumptions *)
+  let initbranch_prenorm = get_initbranch_prenorm () in
+
+  let beforeTrans= Sys.time() in
+(* Branch is an association list from normalized terms to their prenormalized counterparts *)
+  let branch = (List.combine initbranch initbranch_prenorm) in
+  let prenorm_refutation = get_translation branch refutation in
+  let transTime= int_of_float ((Sys.time() -. beforeTrans) *. 1000.0) in
+
+  let (size, _,_,_,_,_,_,_,_) as stats = statistic prenorm_refutation in
+  print_prenorm_ref_result transTime stats prenorm_refutation;
 
 	(*** Output Coq ***)
 
   let beforeCoq= Sys.time() in
-  if (!tstp) then (*** output in tstp format ***)
-    begin
-      ref_tstp c prenorm_refutation
-    end
+  printer size prenorm_refutation;
+  let coqTime= int_of_float ((Sys.time() -. beforeCoq) *. 1000.0) in
+  if !result_coq then if (!verbosity > 3) then Printf.printf  "Coq output done: time %d  \n" coqTime ;
+
+  print_latex_search stats (ref_to_lat initbranch refutation) c;
+  print_latex_translation stats (ref_to_lat initbranch_prenorm prenorm_refutation) c
+
+let coq_proofscript_printer c size prenorm_refutation = 
+  if (!tstp) then ref_tstp c prenorm_refutation
   else
     begin
       if !result_coq then
@@ -252,101 +301,23 @@ let print_coq_proofscript c r =
         if size < max_isabellehol_proofsize then
           ref_isabellehol c prenorm_refutation
         else raise (CoqProofTooBig size) (*FIXME replace with IsabelleHOLProofTooBig*)
-    end;
-  let coqTime= int_of_float ((Sys.time() -. beforeCoq) *. 1000.0) in
-  if !result_coq then if (!verbosity > 3) then Printf.printf  "Coq output done: time %d  \n" coqTime ;
-	(*** Output Latex Search ***)
+    end
 
-  if (result_latex && width < 50 && depth < 30) then
-  Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d \n \n \\begin{tabular}{c} \n %s \\end{tabular} \n\n %%endLatex \n ***)\n" 
-  size depth width (ref_to_lat initbranch refutation)
-  else if (result_latex) then Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d \n \n  %%endLatex \n ***)\n" size depth width;
- 
-	(*** Output Latex Translation ***)
-
-  if (result_latex &&  width < 50 && depth < 30) then
-  Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d cuts %d rewrite  %d NYI %d \n \n \\begin{tabular}{c} \n %s \\end{tabular} \n\n %%endLatex \n***)" 
-  size depth width cut rewrite notyetImpl  (ref_to_lat initbranch_prenorm prenorm_refutation)
-  else if (result_latex) then Printf.fprintf c "(*** \n %%beginLatex \n \n \n Translation successful, probably \n \n size %d depth %d width %d cuts %d rewrite %d  NYI %d \n %%endLatex \n***)" size depth width cut rewrite notyetImpl 
-  ; flush stdout
-  end
-  end		
+let print_coq_proofscript c r =
+  if (!verbosity > vold) then Printf.printf  "starting print_coq_proofscript.\n";
+  print_coq_proof (coq_proofscript_printer c) c r
 
 let print_coq_sproofterm c r =
-  
   if (!verbosity > vold) then Printf.printf "starting print_coq_proofspfterm.\n";
-  if (!verbosity > vold) then print_refut r;
+  let printer size prenorm_refutation =
+    if (!result_coq && size < maxcoqproofsize)
+    then ref_coq_spfterm c prenorm_refutation
+    else raise (CoqProofTooBig size) in
+  print_coq_proof printer c r
 
-	(*** Search ***)
 
-  (* get assumptions *)
-  let initbranch1 = (List.rev !initial_branch) in
-  let initbranch = (List.map preprocess initbranch1) in (*** Chad, Aug 2011: Preprocess to handle LEIBEQ_TO_PRIMEQ here as well as in suche.ml. Example: SYO348^5.p ***)
-  let b =List.map get_literal initbranch in
-  (* call search *)
-  let (refutation,con,searchTime,isTimeout) = search_refutation b r in
-  (* check wether the dependency of the returned refutation is a subset of the assumptions - fail indicates some bug*)
-  if assert_condition && not (Dependency.is_empty (Dependency.diffunion [] [con] [b]) ) then 
-    (if (!verbosity > 0) then Printf.printf  "Error with refutation: Still open conditions  \n") ;	 
-  let (size,_,depth,width,_,cut,rewrite,notyetImpl,timeouts) = statistic refutation in
-  (* if timeout was reached the refutation is unfinished - at least output some information about the completed part*)
-	if isTimeout  || timeouts > 0 then  
-  	  (if (!verbosity > 0) then Printf.printf  "Timeout!  time %d ms size %d depth %d width %d cuts %d rewrite %d NYI %d timeouts %d \n" searchTime size depth width cut rewrite notyetImpl timeouts)
-  	else 	
-  begin
-    if result_print_search then Printf.printf  "%s\n" (ref_str refutation);
-    if (!verbosity > 3) then Printf.printf "Search time %d ms size %d depth %d width %d \n" searchTime size depth width ;
 
-	(*** Translation  ***)
 
-(* get prenormalized assumptions *)
-  let initbranch_prenorm = List.map 
-	(fun pn ->let p= onlynegnorm pn in  if debug_translation then Printf.printf  "%s = %s \n" (trm_str pn) (trm_str p) ;p ) 
-	(List.rev !initial_branch_prenorm) in
-  let beforeTrans= Sys.time() in
-(* Branch is an association list from normalized terms to their prenormalized counterparts *)
-  let branch = (List.combine initbranch  initbranch_prenorm) in
-  if debug_translation then (Printf.printf "About to translate. Branch assoc with prenorm:\n"; List.iter (fun (m,p) -> Printf.printf "trm: %s\npre: %s\n" (trm_str m) (trm_str p)) branch);
-(* The flag pftrm_eager_rewrite decides wether rewrites are eagerly or lazily handled *) 
-  let prenorm_refutation =
-    begin
-      try
-	if pftrm_eager_rewrite then
-	  eager_translate branch refutation
-	else
-	  lazy_translate branch refutation
-      with TranslateFailure -> safe_translate branch refutation (*** Chad Aug 2011 : Fall back on safe_translate ***)
-    end
-  in
-  if debug_translation then (Printf.printf "Translated prenorm_refutation\n%s\n" (ref_str prenorm_refutation));
-  let transTime= int_of_float ((Sys.time() -. beforeTrans) *. 1000.0) in
-  begin
-  let (size,_,depth,width,_,cut,rewrite,notyetImpl,_) = statistic prenorm_refutation  in
-  if result_print_translation then Printf.printf  "%s\n" (ref_str prenorm_refutation);
-  if (!verbosity > 3) then Printf.printf  "Translation NYI %d time %d ms size %d depth %d width %d cuts %d rewrite %d  \n" notyetImpl transTime size depth width cut rewrite  ;
-
-	(*** Output Coq ***)
-
-  let beforeCoq= Sys.time() in
-  if (!result_coq && size < maxcoqproofsize ) then ref_coq_spfterm c prenorm_refutation else raise (CoqProofTooBig size);
-  let coqTime= int_of_float ((Sys.time() -. beforeCoq) *. 1000.0) in
-  if !result_coq then if (!verbosity > 3) then Printf.printf  "Coq output done: time %d  \n" coqTime ;
-	(*** Output Latex Search ***)
-
-  if (result_latex && width < 50 && depth < 30) then
-  Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d \n \n \\begin{tabular}{c} \n %s \\end{tabular} \n\n %%endLatex \n ***)\n" 
-  size depth width (ref_to_lat initbranch refutation)
-  else if (result_latex) then Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d \n \n  %%endLatex \n ***)\n" size depth width;
- 
-	(*** Output Latex Translation ***)
-
-  if (result_latex &&  width < 50 && depth < 30) then
-  Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d cuts %d rewrite  %d NYI %d \n \n \\begin{tabular}{c} \n %s \\end{tabular} \n\n %%endLatex \n***)" 
-  size depth width cut rewrite notyetImpl  (ref_to_lat initbranch_prenorm prenorm_refutation)
-  else if (result_latex) then Printf.fprintf c "(*** \n %%beginLatex \n \n \n Translation successful, probably \n \n size %d depth %d width %d cuts %d rewrite %d  NYI %d \n %%endLatex \n***)" size depth width cut rewrite notyetImpl 
-  ; flush stdout
-  end
-  end		
   
 (*** Chad, August 2011: Added this way of printing proof information at the request of Jasmin Blanchette. ***)
 let print_hocore c r =
@@ -468,13 +439,6 @@ let print_hocore c r =
 	    hocore_1 c r (((norm name_def (neg (Ap(m,Name(x,b))))),name)::a)
 	  with Not_found -> raise (GenericError ("Unknown Assumption " ^ (trm_str (neg (Ap(Forall(b),m))))))
 	end
-    | EqFR(a1,a2,m,n,r) ->  (*** This doesn't appear to be used at the moment. - Chad, Aug 2011 ***)
-	begin
-	  try
-	    let name = List.assoc (Ap(Ap(Eq(Ar(a1,a2)),m),n)) a in
-	    hocore_1 c r (((norm name_def (forall a1 (eq a2 (Ap(m,DB(0,a1))) (Ap(n,DB(0,a1)))))),name)::a)
-	  with Not_found -> raise (GenericError ("Unknown Assumption " ^ (trm_str (Ap(Ap(Eq(Ar(a1,a2)),m),n)))))
-	end
     | NegEqFR(a1,a2,m,n,r) -> 
 	begin
 	  try
@@ -511,7 +475,7 @@ let print_hocore c r =
     end
 
 (*** Chad - July 2012 - output proof script in tstp format. ***)
-let print_tstp c (r : State.refut) =
+let print_tstp c (r : Refut.refut) =
   tstp := true;
 
   (*To conform to the TSTP syntax spec, we first need to print the
@@ -525,15 +489,22 @@ let print_tstp c (r : State.refut) =
       print_endline ("|types| = " ^ string_of_int (List.length types))
     end;
 
-  List.map (fun basetype_name ->
+  List.iter (fun basetype_name ->
+    Hashtbl.add coq_used_names basetype_name (); (* Chad, May 3 2016: mark this as used to avoid conflicts if new names are created *)
     output_string c
       ("thf(ty" ^ basetype_name ^ ", type, " ^ basetype_name ^ " : $tType).\n"))
    types;
 
-  List.map (fun (name, stp) ->
+  List.iter (fun (name, stp) ->
     let tstp_name = tstpizename name in
+    Hashtbl.add coq_used_names tstp_name (); (* Chad, May 3 2016: mark this as used to avoid conflicts if new names are created *)
     output_string c ("thf(ty" ^ tstp_name ^ ", type, " ^ tstp_name ^ " : ");
     Coq.print_stp_tstp stdout stp true;
     output_string c (").\n"))
-   consts;
+    consts;
   print_coq_proofscript c r
+
+
+let refut_trms r =
+  let (refutation,_,_,_) = get_refutation (get_initbranch ()) r in
+  ref_trms refutation
