@@ -1,4 +1,4 @@
-open State
+open Refut
 
 val print_coq_proofscript : out_channel -> refut -> unit
 
@@ -8,4 +8,4 @@ val print_coq_sproofterm : out_channel -> refut -> unit
 
 val print_hocore : out_channel -> refut -> unit
 
-
+val refut_trms : refut -> Syntax.trm list
