@@ -1,5 +1,7 @@
 open Flags
 open State
+open Atom
+open Refut
 open String
 open Syntax
 open Refutation
@@ -30,7 +32,7 @@ let sort_steps sl litc = List.stable_sort
 		if c = 0 then Pervasives.compare (Step.heuristic b litc ) (Step.heuristic a litc) else c ) sl 
 (* stable_sort keeps the order of the exists*)
 
-(** Input: List of clasues cl,cw and clause to rule info cr 
+(** Input: List of clauses cl,cw and clause to rule info cr
 	Iterates over the list of clauses and adds symmetric clauses in certain cases, while avoiding duplication.
 	**)
 let rec add_sym_clauses cl cw cr =
@@ -842,18 +844,6 @@ let rec process_refut1 b r =
 	let con = Dependency.diffunion [get_literal h] [con] [[s']] in
     	(NegAll(h,s,r,a,m,x),con)
 
-  | EqFR(a,a',s,t,r1) ->   if debug_search then Printf.printf  "apply EqFR\n";
-    	let h =  (eq (Ar(a,a')) s t) in
-    	let m = (norm name_def (forall a (eq a' (Ap(s,DB(0,a))) (Ap(t,DB(0,a))) ))) in
-	let s' = get_literal m in
-	let (r,con)=try process_refut [s'] b r1 with Independent_Refutation(r,con)-> (r,con) in
-	if (get_bool_flag "PFTRM_REMOVE_INDEPENDENT") && pftrm_Independent_refutation  && not (Dependency.mem s' con) then
-		( if debug_Independent_refutation then Printf.printf  "found Independent refutation\n";
-			raise (Independent_Refutation(r,con)) )
-	else 
-	let con = Dependency.diffunion [get_literal h] [con] [[s']] in
-    	(EqualFunc(h,m,r),con)
-
   | NegEqFR(a,a',s,t,r1) -> if debug_search then Printf.printf  "apply NegEqFR\n";
     	let h = neg (eq (Ar(a,a')) s t) in
     	let m = normneg (norm name_def (forall a (eq a' (Ap(s,DB(0,a))) (Ap(t,DB(0,a))) ))) in
