@@ -4,6 +4,7 @@ open Syntax
 open Refutation
 open Flag
 open Norm
+open Error
 
 exception Mating_Mismatch of string
 
@@ -1000,7 +1001,8 @@ let rec safe_fill_translate r =
       begin
 	match (tpof pred) with
 	| Ar(a,Prop) ->
-	    let m1 = onlybetanorm (Ap(pred,Ap(eps,pred))) in
+	    (* TODO: why was m1 ever declared here? *)
+	    (*let m1 = onlybetanorm (Ap(pred,Ap(eps,pred))) in*)
 	    let m2 = onlybetanorm (Ap(shift pred 1 0,DB(0,a))) in
 	    let m2n = neg m2 in
 	    let m3 = forall a m2n in
