@@ -160,7 +160,7 @@ let rec trm_str_rec m p =
   match m with
     Name(x,_) -> x
   | Lam(Ar(a1,a2),DB(0,_)) when (a1 = a2) -> "#1:" ^ (stp_str a1) (*** Church Numeral 1 Printed in a special way. - Chad, Feb 2, 2011 ***)
-  | Lam(Ar(a1,a2),Lam(a3,m)) when ((a1 = a2) & (a1 = a3) & (church_num_body m)) -> "#" ^ (string_of_int (church_num_body_val m)) ^ ":" ^ (stp_str a1) (*** Church Numerals Printed in a special way. - Chad, Feb 2, 2011 ***)
+  | Lam(Ar(a1,a2),Lam(a3,m)) when a1 = a2 && a1 = a3 && church_num_body m -> "#" ^ string_of_int (church_num_body_val m) ^ ":" ^ stp_str a1 (*** Church Numerals Printed in a special way. - Chad, Feb 2, 2011 ***)
   | False -> "False"
   | Imp -> "imp"
   | Forall(a) -> "Pi:" ^ (stp_str a)
@@ -2142,7 +2142,7 @@ let rec trm_to_isar c m bound =
 	        trm_to_isar c m2 bound;
 	        Printf.fprintf c ")";
 	      end
-    | _ -> raise (GenericSyntaxError ("Unknown case in trm_to_isar version : " ^ (trm_str m)))
+
  (* Prints consecutive lambda-terms as a single fun in Isar. *)
 and print_lam_isar c a m bound =
 	let bound = Variables.push bound in
@@ -2210,3 +2210,46 @@ let rec base_types_of_trm acc : trm -> string list = function
   | Forall stp | Eq stp | Choice stp | Exists stp | DB (_, stp) ->
     base_types acc stp
   | False | Imp | True | Neg | Or | And | Iff -> acc
+
+
+let fresh_prefix = "__"
+
+let make_fresh_name i = fresh_prefix ^ string_of_int i
+let is_fresh_name = Utils.string_starts_with fresh_prefix
+
+let rec map_names f = function
+  Name (x,a) -> f x a
+| Lam (t,m) -> Lam(t, map_names f m)
+| Ap (m,n) -> Ap(map_names f m, map_names f n)
+| m -> m
+
+let rec fold_names f acc = function
+  Name (x,a) -> f acc x a
+| Lam (t,m) -> let (acc', m') = fold_names f acc m in (acc', Lam(t, m'))
+| Ap (m,n) ->
+    let (acc' , m') = fold_names f acc  m in
+    let (acc'', n') = fold_names f acc' n in
+    (acc'', Ap(m', n'))
+| m -> (acc, m)
+
+let normalize_fresh_uni =
+  map_names (fun x a -> Name((if is_fresh_name x then fresh_prefix else x), a))
+
+let normalize_asc_fun p name ((m,i) as acc) x a =
+  if p x
+  then
+  begin
+    let (acc', x') =
+      try (acc, List.assoc x m)
+      with Not_found -> let i' = i+1 in (((x,i')::m,i'), i')
+    in (acc', Name(name x', a))
+  end
+  else (acc, Name (x,a))
+
+let normalize_fresh_asc m =
+  fold_names (normalize_asc_fun is_fresh_name make_fresh_name) ([], 0) m
+
+let normalize_asc m =
+  fold_names (normalize_asc_fun (fun x -> true) (fun i -> "c" ^ string_of_int i)) ([], 0) m
+
+let contains_fresh_names m = let ((_, i), _) = normalize_fresh_asc m in i > 0
