@@ -147,3 +147,9 @@ end
 val consts_of_trm : ctx -> trm -> ctx
 val base_types : string list -> stp -> string list
 val base_types_of_trm : string list -> trm -> string list
+
+val make_fresh_name : int -> string
+val normalize_fresh_uni : trm -> trm
+val normalize_fresh_asc : trm -> ((string * int) list * int) * trm
+val normalize_asc : trm -> ((string * int) list * int) * trm
+val contains_fresh_names : trm -> bool
